import urllib, sys, os, shutil, re, time, xbmcaddon, xbmcplugin, xbmcgui, xbmc, pickle
MODE_SEARCH_SONGS = 1
MODE_SEARCH_ALBUMS = 2
MODE_SEARCH_ARTISTS = 3
MODE_SEARCH_ARTISTS_ALBUMS = 4
MODE_SEARCH_PLAYLISTS = 5
MODE_ARTIST_POPULAR = 6
MODE_POPULAR_SONGS = 7
MODE_FAVORITES = 8
MODE_PLAYLISTS = 9
MODE_ALBUM = 10
MODE_ARTIST = 11
MODE_PLAYLIST = 12
MODE_SONG_PAGE = 13
MODE_SIMILAR_ARTISTS = 14
MODE_SONG = 15
MODE_FAVORITE = 16
MODE_UNFAVORITE = 17
MODE_MAKE_PLAYLIST = 18
MODE_REMOVE_PLAYLIST = 19
MODE_RENAME_PLAYLIST = 20
MODE_REMOVE_PLAYLIST_SONG = 21
MODE_ADD_PLAYLIST_SONG = 22

ACTION_MOVE_LEFT = 1
ACTION_MOVE_UP = 3
ACTION_MOVE_DOWN = 4
ACTION_PAGE_UP = 5
ACTION_PAGE_DOWN = 6
ACTION_SELECT_ITEM = 7
ACTION_PREVIOUS_MENU = 10

# Formats for track labels
ARTIST_ALBUM_NAME_LABEL = 0
NAME_ALBUM_ARTIST_LABEL = 1

baseDir = os.getcwd()
resDir = xbmc.translatePath(os.path.join(baseDir, 'resources'))
libDir = xbmc.translatePath(os.path.join(resDir,  'lib'))
imgDir = xbmc.translatePath(os.path.join(resDir,  'img'))
cacheDir = os.path.join(xbmc.translatePath('special://masterprofile/addon_data/'), os.path.basename(os.getcwd()))
thumbDirName = 'thumb'
thumbDir = os.path.join('special://masterprofile/addon_data/', os.path.basename(os.getcwd()), thumbDirName)

baseModeUrl = 'plugin://plugin.audio.groove/'
playlistUrl = baseModeUrl + '?mode=' + str(MODE_PLAYLIST)
playlistsUrl = baseModeUrl + '?mode=' + str(MODE_PLAYLISTS)
favoritesUrl = baseModeUrl + '?mode=' + str(MODE_FAVORITES)

searchArtistsAlbumsName = "Search for artist's albums..."

thumbDef = os.path.join(imgDir, 'default.tbn')
listBackground = os.path.join(imgDir, 'listbackground.png')

sys.path.append (libDir)
from GroovesharkAPI import GrooveAPI
from GroovesharkAPI import GrooveAPIv1

try:
    groovesharkApi = GrooveAPI()
    if groovesharkApi.pingService() != True:
        raise StandardError('No Grooveshark service')
except:
    dialog = xbmcgui.Dialog()
    dialog.ok('Grooveshark XBMC', 'Unable to connect with Grooveshark.', 'Please try again later')
    sys.exit(-1)
  

class _Info:
    def __init__( self, *args, **kwargs ):
        self.__dict__.update( kwargs )

# Window dialog to select a grooveshark playlist        
class GroovesharkPlaylistSelect(xbmcgui.WindowDialog):
    
    def __init__(self, items=[]):
        gap = int(self.getHeight()/100)
        w = int(self.getWidth()*0.5)
        h = self.getHeight()-30*gap
        rw = self.getWidth()
        rh = self.getHeight()
        x = rw/2 - w/2
        y = rh/2 -h/2
        
        self.imgBg = xbmcgui.ControlImage(x+gap, 5*gap+y, w-2*gap, h-5*gap, listBackground)
        self.addControl(self.imgBg)

        self.playlistControl = xbmcgui.ControlList(2*gap+x, y+3*gap+30, w-4*gap, h-10*gap, textColor='0xFFFFFFFF', selectedColor='0xFFFF4242', itemTextYOffset=0, itemHeight=50, alignmentY = 0)
        self.addControl(self.playlistControl)

        self.lastPos = 0
        self.isSelecting = False
        self.selected = -1
        listitems = []
        for playlist in items:
            listitems.append(xbmcgui.ListItem(playlist[0]))
        listitems.append(xbmcgui.ListItem('New...'))
        self.playlistControl.addItems(listitems)
        self.setFocus(self.playlistControl)
        self.playlistControl.selectItem(0)
        item = self.playlistControl.getListItem(self.lastPos)
        item.select(True)

    # Highlight selected item
    def setHighlight(self):
        if self.isSelecting:
            return
        else:
            self.isSelecting = True
        
        pos = self.playlistControl.getSelectedPosition()
        if pos >= 0:
            item = self.playlistControl.getListItem(self.lastPos)
            item.select(False)
            item = self.playlistControl.getListItem(pos)
            item.select(True)
            self.lastPos = pos
        self.isSelecting = False

    # Control - select
    def onControl(self, control):
        if control == self.playlistControl:
            self.selected = self.playlistControl.getSelectedPosition()
            self.close()

    # Action - close or up/down        
    def onAction(self, action):
        if action == ACTION_PREVIOUS_MENU:
            self.selected = -1
            self.close()
        elif action == ACTION_MOVE_UP or action == ACTION_MOVE_DOWN or action == ACTION_PAGE_UP or action == ACTION_PAGE_DOWN == 6:
            self.setFocus(self.playlistControl)
            self.setHighlight()
     
class Groveshark:
    
    albumImg = xbmc.translatePath(os.path.join(imgDir, 'album.png'))
    artistImg = xbmc.translatePath(os.path.join(imgDir, 'artist.png'))
    artistsAlbumsImg = xbmc.translatePath(os.path.join(imgDir, 'artistsalbums.png'))
    favoritesImg = xbmc.translatePath(os.path.join(imgDir, 'favorites.png'))
    playlistImg = xbmc.translatePath(os.path.join(imgDir, 'playlist.png'))
    usersplaylistsImg = xbmc.translatePath(os.path.join(imgDir, 'usersplaylists.png'))
    popularSongsImg = xbmc.translatePath(os.path.join(imgDir, 'popularSongs.png'))
    popularSongsArtistImg = xbmc.translatePath(os.path.join(imgDir, 'popularSongsArtist.png'))
    songImg = xbmc.translatePath(os.path.join(imgDir, 'song.png'))
    defImg = xbmc.translatePath(os.path.join(imgDir, 'default.tbn'))
    fanImg = xbmc.translatePath(os.path.join(baseDir, 'fanart.png'))

    settings = xbmcaddon.Addon(id='plugin.audio.groove')
    songsearchlimit = int(settings.getSetting('songsearchlimit'))
    albumsearchlimit = int(settings.getSetting('albumsearchlimit'))
    artistsearchlimit = int(settings.getSetting('artistsearchlimit'))
    songspagelimit = int(settings.getSetting('songspagelimit'))
    username = settings.getSetting('username')
    password = settings.getSetting('password')
    sessionidv1 = settings.getSetting('sessionidv1')
    userid = 0
    
    def __init__( self ):
        self._handle = int(sys.argv[1])
        if os.path.isdir(cacheDir) == False:
            os.makedirs(cacheDir)
            xbmc.log("Made " + cacheDir)
        artDir = xbmc.translatePath(thumbDir)
        if os.path.isdir(artDir) == False:
            os.makedirs(artDir)
            xbmc.log("Made " + artDir)
            
        self.groovesharkApiv1 = GrooveAPIv1(self.sessionidv1)
            
    # Top-level menu
    def categories(self):

        self.userid = self._get_login()
        
        # Setup
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
        
        self._add_dir('Search for songs...', '', MODE_SEARCH_SONGS, self.songImg, 0)
        self._add_dir('Search for albums...', '', MODE_SEARCH_ALBUMS, self.albumImg, 0)
        self._add_dir('Search for artists...', '', MODE_SEARCH_ARTISTS, self.artistImg, 0)
        self._add_dir(searchArtistsAlbumsName, '', MODE_SEARCH_ARTISTS_ALBUMS, self.artistsAlbumsImg, 0)
        self._add_dir("Search for user's playlists...", '', MODE_SEARCH_PLAYLISTS, self.usersplaylistsImg, 0)
        self._add_dir('Popular songs for artist...', '', MODE_ARTIST_POPULAR, self.popularSongsArtistImg, 0)
        self._add_dir('Popular songs', '', MODE_POPULAR_SONGS, self.popularSongsImg, 0)
        if (self.userid != 0):
            self._add_dir('My favorites', '', MODE_FAVORITES, self.favoritesImg, 0)
            self._add_dir('My playlists', '', MODE_PLAYLISTS, self.playlistImg, 0)

    # Search for songs            
    def searchSongs(self):
        query = self._get_keyboard(default="", heading="Search for songs")
        if (query != ''):
            songs = groovesharkApi.getSongSearchResults(query, limit = self.songsearchlimit)
            if (len(songs) > 0):
                self._add_songs_directory(songs)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching songs.')
                self.categories()
        else:
            self.categories()
    
    # Search for albums
    def searchAlbums(self):
        query = self._get_keyboard(default="", heading="Search for albums")
        if (query != ''): 
            albums = groovesharkApi.getAlbumSearchResults(query, limit = self.albumsearchlimit)
            if (len(albums) > 0):
                self._add_albums_directory(albums)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching albums.')
                self.categories()
        else:
            self.categories()
    
    # Search for artists
    def searchArtists(self):
        query = self._get_keyboard(default="", heading="Search for artists")
        if (query != ''): 
            artists = groovesharkApi.getArtistSearchResults(query, limit = self.artistsearchlimit)
            if (len(artists) > 0):
                self._add_artists_directory(artists)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching artists.')
                self.categories()
        else:
            self.categories()

    # Search for playlists
    def searchPlaylists(self):
        query = self._get_keyboard(default="", heading="Username")
        if (query != ''): 
            playlists = groovesharkApi.getUserPlaylistsEx(query)
            if (len(playlists) > 0):
                self._add_playlists_directory(playlists)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No Grooveshark playlists found.')
                self.categories()
        else:
            self.categories()

    # Search for artists albums
    def searchArtistsAlbums(self, artistName = None):
        if artistName == None or artistName == searchArtistsAlbumsName:
            query = self._get_keyboard(default="", heading="Search for artist's albums")
        else:
            query = artistName
        if (query != ''): 
            artists = groovesharkApi.getArtistSearchResults(query, limit = self.artistsearchlimit)
            if (len(artists) > 0):
                artist = artists[0]
                artistID = artist[1]
                xbmc.log("Found " + artist[0] + "...")
                albums = groovesharkApi.getArtistAlbums(artistID, limit = self.albumsearchlimit)
                if (len(albums) > 0):
                    self._add_albums_directory(albums, artistID)
                else:
                    dialog = xbmcgui.Dialog()
                    dialog.ok('Grooveshark XBMC', 'No matching albums.')
                    self.categories()
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching artists.')
                self.categories()
        else:
            self.categories()
                  
    # Get my favorites
    def favorites(self):
        userid = self._get_login()
        if (userid != 0):
            favorites = groovesharkApi.getUserFavoriteSongs()
            if (len(favorites) > 0):
                self._add_songs_directory(favorites, isFavorites=True)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You have no favorites.')
                self.categories()
    
    # Get popular songs
    def popularSongs(self):
        popular = groovesharkApi.getPopularSongsToday(limit = self.songsearchlimit)
        if (len(popular) > 0):
            self._add_songs_directory(popular)
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'No popular songs.')
            self.categories()

    # Get my playlists             
    def playlists(self):
        userid = self._get_login()
        if (userid != 0):
            playlists = groovesharkApi.getUserPlaylists()
            if (len(playlists) > 0):
                self._add_playlists_directory(playlists)
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You have no Grooveshark playlists.')
                self.categories()
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in ', ' to see your Grooveshark playlists.')
                
    # Make songs a favorite 
    def favorite(self, songid):
        userid = self._get_login()
        if (userid != 0):
            xbmc.log("Favorite song: " + str(songid))
            groovesharkApi.addUserFavoriteSong(songID = songid)
            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Added to favorites, 1000, ' + thumbDef + ')')
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to add favorites.')
            
    # Remove song from favorites
    def unfavorite(self, songid, prevMode=0):
        userid = self._get_login(version = 1)
        if (userid != 0):
            xbmc.log("Unfavorite song: " + str(songid) + ', previous mode was ' + str(prevMode))
            self.groovesharkApiv1.unfavorite(songID = songid)
            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Removed from Grooveshark favorites, 1000, ' + thumbDef + ')')
            # Refresh to remove item from directory
            if (int(prevMode) == MODE_FAVORITES):
                xbmc.executebuiltin("Container.Refresh(" + favoritesUrl + ")")
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to remove Grooveshark favorites.')
            

    # Show selected album
    def album(self, albumid):
        album = groovesharkApi.getAlbumSongs(albumid, limit = self.songsearchlimit)
        self._add_songs_directory(album, trackLabelFormat=NAME_ALBUM_ARTIST_LABEL)

    # Show selected artist
    def artist(self, artistid):
        albums = groovesharkApi.getArtistAlbums(artistid, limit = self.albumsearchlimit)
        self._add_albums_directory(albums, artistid)
    
    # Show selected playlist
    def playlist(self, playlistid, playlistname, version = 2):
        userid = self._get_login()
        if (userid != 0):
            if version == 1:
                songs = self.groovesharkApiv1.playlistGetSongs(playlistid)                
            else:
                songs = groovesharkApi.getPlaylistSongs(playlistid)
            self._add_songs_directory(songs, trackLabelFormat=NAME_ALBUM_ARTIST_LABEL, playlistid=playlistid, playlistname=playlistname)
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to get Grooveshark playlists.')
            
    # Show popular songs of the artist
    def artistPopularSongs(self):
        query = self._get_keyboard(default="", heading="Artist")
        if (query != ''): 
            artists = groovesharkApi.getArtistSearchResults(query, limit = self.artistsearchlimit)
            if (len(artists) > 0):
                artist = artists[0]
                artistID = artist[1]
                xbmc.log("Found " + artist[0] + "...")
                songs = groovesharkApi.getArtistPopularSongs(artistID, limit = self.songsearchlimit)
                if (len(songs) > 0):
                    self._add_songs_directory(songs, trackLabelFormat=NAME_ALBUM_ARTIST_LABEL)
                else:
                    dialog = xbmcgui.Dialog()
                    dialog.ok('Grooveshark XBMC', 'No popular songs.')
                    self.categories()
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'No matching artists.')
                self.categories()
        else:
            self.categories()
            
    # Play a song
    def playSong(self, item):
        songid = item.getProperty('songid')
        song = groovesharkApi.getSongURLFromSongID(songid)
        if song != '':
            item.setPath(song)
            xbmc.log("Playing: " + song)
            xbmcplugin.setResolvedUrl(handle=int(sys.argv[1]), succeeded=True, listitem=item)
        else:
            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Unable to play song, 1000, ' + thumbDef + ')')
    
    # Make a song directory item
    def songItem(self, songid, name, album, artist, coverart, trackLabelFormat=ARTIST_ALBUM_NAME_LABEL):
        songImg = self._get_icon(coverart, 'song-' + str(songid) + "-image")
        if int(trackLabelFormat) == NAME_ALBUM_ARTIST_LABEL:
            trackLabel = name + " - " + album + " - " + artist
        else:
            trackLabel = artist + " - " + album + " - " + name
        item = xbmcgui.ListItem(label = trackLabel, thumbnailImage=songImg, iconImage=songImg)
        item.setInfo( type="music", infoLabels={ "title": name, "album": album, "artist": artist} )
        item.setProperty('mimetype', 'audio/mpeg')
        item.setProperty("IsPlayable", "true")
        item.setProperty('songid', str(songid))
        item.setProperty('coverart', songImg)
        item.setProperty('title', name)
        item.setProperty('album', album)
        item.setProperty('artist', artist)
        
        return item
    
    # Next page of songs
    def songPage(self, page, trackLabelFormat, playlistid = 0, playlistname = ''):
        self._add_songs_directory([], trackLabelFormat, page, playlistid = playlistid, playlistname = playlistname)
        
    # Make a playlist from an album      
    def makePlaylist(self, albumid, name):
        userid = self._get_login(version = 1)
        if (userid != 0):
            re.split(' - ',name,1)
            nameTokens = re.split(' - ',name,1) # suggested name
            name = self._get_keyboard(default=nameTokens[0], heading="Grooveshark playlist name")
            if name != '':
                album = groovesharkApi.getAlbumSongs(albumid, limit = self.songsearchlimit)
                songids = []
                for song in album:
                    songids.append(song[1])
                if self.groovesharkApiv1.playlistCreateUnique(name, songids) == 0:
                    dialog = xbmcgui.Dialog()
                    dialog.ok('Grooveshark XBMC', 'Cannot create Grooveshark playlist ', name)
                else:
                    xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Grooveshark playlist created, 1000, ' + thumbDef + ')')
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in ', ' to create a Grooveshark playlist.')
    
    # Rename a playlist
    def renamePlaylist(self, playlistid, name):
        userid = self._get_login(version = 1)
        if (userid != 0):
            newname = self._get_keyboard(default=name, heading="Grooveshark playlist name")
            if newname == '':
                return
            elif self.groovesharkApiv1.playlistRename(playlistid, newname) == 0:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'Cannot rename Grooveshark playlist ', name)
            else:
                # Refresh to show new item name
                xbmc.executebuiltin("Container.Refresh")
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in ', ' to rename a Grooveshark playlist.')
        
    # Remove a playlist
    def removePlaylist(self, playlistid, name):
        dialog = xbmcgui.Dialog()
        if dialog.yesno('Grooveshark XBMC', name, 'Delete this Grooveshark playlist?') == True:
            userid = self._get_login(version = 1)
            if (userid != 0):
                if self.groovesharkApiv1.playlistDelete(playlistid) == 0:
                    dialog = xbmcgui.Dialog()
                    dialog.ok('Grooveshark XBMC', 'Cannot remove Grooveshark playlist ', name)
                else:
                    # Refresh to remove item from directory
                    xbmc.executebuiltin("Container.Refresh(" + playlistsUrl + ")")
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You must be logged in ', ' to delete a Grooveshark playlist.')

    # Add song to playlist
    def addPlaylistSong(self, songid):
        userid = self._get_login(version = 1)
        if (userid != 0):
            playlists = groovesharkApi.getUserPlaylists()
            if (len(playlists) > 0):
                ret = 0
                # Select the playlist
                playlistSelect = GroovesharkPlaylistSelect(items=playlists)
                playlistSelect.setFocus(playlistSelect.playlistControl)
                playlistSelect.doModal()
                i = playlistSelect.selected
                del playlistSelect
                if i > -1:
                    # Add a new playlist
                    if i >= len(playlists):
                        name = self._get_keyboard(default='', heading="Grooveshark playlist name")
                        if name != '':
                            songIds = []
                            songIds.append(songid)
                            if self.groovesharkApiv1.playlistCreateUnique(name, songIds) == 0:
                                dialog = xbmcgui.Dialog()
                                dialog.ok('Grooveshark XBMC', 'Cannot create Grooveshark playlist ', name)
                            else:
                                xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Grooveshark playlist created, 1000, ' + thumbDef + ')')
                    # Existing playlist
                    else:
                        playlist = playlists[i]
                        playlistid = playlist[1]
                        xbmc.log("Add song " + str(songid) + " to playlist " + str(playlistid))
                        ret = self.groovesharkApiv1.playlistAddSong(playlistid, songid, 0)
                        if ret == 0:
                            dialog = xbmcgui.Dialog()
                            dialog.ok('Grooveshark XBMC', 'Cannot add to playlist ')
                        else:    
                            xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Added song to Grooveshark playlist, 1000, ' + thumbDef + ')')
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You have no Grooveshark playlists.')
                self.categories()
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to add a song to a Grooveshark playlist.')

    # Remove song from playlist
    def removePlaylistSong(self, playlistid, playlistname, songpos):
        dialog = xbmcgui.Dialog(version = 1)
        if dialog.yesno('Grooveshark XBMC', 'Delete this song from the Grooveshark playlist?') == True:
            userid = self._get_login(version = 1)
            if (userid != 0):
                if self.groovesharkApiv1.playlistDeleteSong(playlistid, songpos) == 0:
                    dialog = xbmcgui.Dialog()
                    dialog.ok('Grooveshark XBMC', 'Failed to remove', 'song from Grooveshark playlist.')
                else:
                    # Refresh to remove item from directory
                    xbmc.executebuiltin('XBMC.Notification(Grooveshark XBMC, Removed song from Grooveshark playlist, 1000, ' + thumbDef + ')')
                    xbmc.executebuiltin("Container.Update(" + playlistUrl + "&id="+str(playlistid) + "&name=" + playlistname + "&version=1)")
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'You must be logged in', 'to delete a song from a Grooveshark playlist.')
      
    # Find similar artists to searched artist
    def similarArtists(self, artistId):
        similar = self.groovesharkApiv1.artistGetSimilar(artistId, limit = self.artistsearchlimit)
        if (len(similar) > 0):
            self._add_artists_directory(similar)
        else:
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'No similar artists.')
            self.categories()
        
    # Get keyboard input
    def _get_keyboard(self, default="", heading="", hidden=False):
        kb = xbmc.Keyboard(default, heading, hidden)
        kb.doModal()
        if (kb.isConfirmed()):
            return unicode(kb.getText(), "utf-8")
        return ''
    
    # Login to grooveshark
    def _get_login(self, version = 2):
        if (self.username == "" or self.password == ""):
            dialog = xbmcgui.Dialog()
            dialog.ok('Grooveshark XBMC', 'Unable to login.', 'Check username and password in settings.')
            return 0
        else:
            if self.userid == 0:
                if version == 1:
                    uid = self.groovesharkApiv1.login(self.username, self.password)
                else:
                    uid = groovesharkApi.login(self.username, self.password)
            if (uid != 0):
                return uid
            else:
                dialog = xbmcgui.Dialog()
                dialog.ok('Grooveshark XBMC', 'Unable to login.', 'Check username and password in settings.')
                return 0
    
    # Get a song directory item
    def _get_song_item(self, song, trackLabelFormat):
        name = song[0]
        songid = song[1]
        album = song[2]
        artist = song[4]
        coverart = song[6]
        return self.songItem(songid, name, album, artist, coverart, trackLabelFormat)            
        
    # File download            
    def _get_icon(self, url, songid):
        if url != 'None':
            localThumb = os.path.join(xbmc.translatePath(os.path.join(thumbDir, str(songid)))) + '.tbn'
            try:
                if os.path.isfile(localThumb) == False:
                    loc = urllib.URLopener()
                    loc.retrieve(url, localThumb)
            except:
                shutil.copy2(thumbDef, localThumb)
            return os.path.join(os.path.join(thumbDir, str(songid))) + '.tbn'
        else:
            return thumbDef
    
    # Add songs to directory
    def _add_songs_directory(self, songs, trackLabelFormat=ARTIST_ALBUM_NAME_LABEL, page=0, playlistid=0, playlistname='', isFavorites=False):

        totalSongs = len(songs)
        page = int(page)

        # No pages needed
        if page == 0 and totalSongs <= self.songspagelimit:
            xbmc.log("Found " + str(totalSongs) + " songs...")
        # Pages
        else:
            # Cache all songs
            if page == 0:
                self._setSavedSongs(songs)
            else:
                songs = self._getSavedSongs()
                totalSongs = len(songs)
                
            if totalSongs > 0:
                start = page * self.songspagelimit
                end = start + self.songspagelimit
                songs = songs[start:end]
        
        id = 0
        for song in songs:
            item = self._get_song_item(song, trackLabelFormat)
            coverart = item.getProperty('coverart')
            songname = song[0]
            songid = song[1]
            songalbum = song[2]
            songartist = song[4]
            u=sys.argv[0]+"?mode="+str(MODE_SONG)+"&name="+urllib.quote_plus(songname)+"&id="+str(songid) \
            +"&album="+urllib.quote_plus(songalbum) \
            +"&artist="+urllib.quote_plus(songartist) \
            +"&coverart="+urllib.quote_plus(coverart)
            fav=sys.argv[0]+"?mode="+str(MODE_FAVORITE)+"&name="+urllib.quote_plus(songname)+"&id="+str(songid)
            unfav=sys.argv[0]+"?mode="+str(MODE_UNFAVORITE)+"&name="+urllib.quote_plus(songname)+"&id="+str(songid)+"&prevmode="
            menuItems = []
            if isFavorites == True:
                unfav = unfav +str(MODE_FAVORITES)
            else:
                menuItems.append(("Grooveshark favorite", "XBMC.RunPlugin("+fav+")"))
            if self.sessionidv1 != '':
                menuItems.append(("Not Grooveshark favorite", "XBMC.RunPlugin("+unfav+")"))
                if playlistid > 0:
                    rmplaylstsong=sys.argv[0]+"?playlistid="+str(playlistid)+"&id="+str(id+1)+"&mode="+str(MODE_REMOVE_PLAYLIST_SONG)+"&name="+playlistname
                    menuItems.append(("Remove from Grooveshark playlist", "XBMC.RunPlugin("+rmplaylstsong+")"))
                else:
                    addplaylstsong=sys.argv[0]+"?id="+str(songid)+"&mode="+str(MODE_ADD_PLAYLIST_SONG)
                    menuItems.append(("Add to Grooveshark playlist", "XBMC.RunPlugin("+addplaylstsong+")"))
            item.addContextMenuItems(menuItems, replaceItems=False)
            xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=item,isFolder=False, totalItems=len(songs))
            id = id + 1

        page = page + 1
        if totalSongs > page * self.songspagelimit:
            u=sys.argv[0]+"?mode="+str(MODE_SONG_PAGE)+"&id=playlistid"+"&page="+str(page)+"&label="+str(trackLabelFormat)+"&name="+playlistname
            self._add_dir('More songs...', u, MODE_SONG_PAGE, self.songImg, 0, totalSongs - (page * self.songspagelimit))

        xbmcplugin.setContent(self._handle, 'songs')
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
    
    # Add albums to directory
    def _add_albums_directory(self, albums, artistid=0):
        n = len(albums)
        xbmc.log("Found " + str(n) + " albums...")
        i = 0
        while i < n:
            album = albums[i]
            albumArtistName = album[0]
            albumName = album[2]
            albumID = album[3]
            albumImage = self._get_icon(album[4], 'album-' + str(albumID))
            self._add_dir(albumName + " - " + albumArtistName, '', MODE_ALBUM, albumImage, albumID, n)
            i = i + 1
        if artistid > 0 and self.sessionidv1 != '':
            self._add_dir('Similar artists...', '', MODE_SIMILAR_ARTISTS, self.artistImg, artistid)
        xbmcplugin.setContent(self._handle, 'albums')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_ALBUM_IGNORE_THE)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
    
    # Add artists to directory
    def _add_artists_directory(self, artists):
        n = len(artists)
        xbmc.log("Found " + str(n) + " artists...")
        i = 0
        while i < n:
            artist = artists[i]
            artistName = artist[0]
            artistID = artist[1]
            self._add_dir(artistName, '', MODE_ARTIST, self.artistImg, artistID, n)
            i = i + 1
        xbmcplugin.setContent(self._handle, 'artists')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_ARTIST_IGNORE_THE)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)

    # Add playlists to directory          
    def _add_playlists_directory(self, playlists):
        n = len(playlists)
        xbmc.log("Found " + str(n) + " playlists...")
        i = 0
        while i < n:
            playlist = playlists[i]
            playlistName = playlist[0]
            playlistID = playlist[1]
            dir = self._add_dir(playlistName, '', MODE_PLAYLIST, self.playlistImg, playlistID, n)
            i = i + 1  
        xbmcplugin.setContent(self._handle, 'files')
        xbmcplugin.addSortMethod(self._handle, xbmcplugin.SORT_METHOD_LABEL)
        xbmcplugin.setPluginFanart(int(sys.argv[1]), self.fanImg)
      
    # Add whatever directory
    def _add_dir(self, name, url, mode, iconimage, id, items=1):

        if url == '':
            u=sys.argv[0]+"?mode="+str(mode)+"&name="+urllib.quote_plus(name)+"&id="+str(id)
        else:
            u = url
        dir=xbmcgui.ListItem(name, iconImage=iconimage, thumbnailImage=iconimage)
        dir.setInfo( type="Music", infoLabels={ "title": name } )
        
        # Custom menu items
        if self.sessionidv1 != '':
            menuItems = []
            if mode == MODE_ALBUM:
                mkplaylst=sys.argv[0]+"?mode="+str(MODE_MAKE_PLAYLIST)+"&name="+name+"&id="+str(id)
                menuItems.append(("Make Grooveshark playlist", "XBMC.RunPlugin("+mkplaylst+")"))
            if mode == MODE_PLAYLIST:
                rmplaylst=sys.argv[0]+"?mode="+str(MODE_REMOVE_PLAYLIST)+"&name="+urllib.quote_plus(name)+"&id="+str(id)
                menuItems.append(("Delete Grooveshark playlist", "XBMC.RunPlugin("+rmplaylst+")"))
                mvplaylst=sys.argv[0]+"?mode="+str(MODE_RENAME_PLAYLIST)+"&name="+urllib.quote_plus(name)+"&id="+str(id)
                menuItems.append(("Rename Grooveshark playlist", "XBMC.RunPlugin("+mvplaylst+")"))
            dir.addContextMenuItems(menuItems, replaceItems=False)
        
        return xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=dir,isFolder=True, totalItems=items)
    
    def _getSavedSongs(self):
        path = os.path.join(cacheDir, 'songs.dmp')
        try:
            f = open(path, 'rb')
            songs = pickle.load(f)
            f.close()
        except:
            songs = []
            pass
        return songs

    def _setSavedSongs(self, songs):            
        try:
            # Create the 'data' directory if it doesn't exist.
            if not os.path.exists(cacheDir):
                os.makedirs(cacheDir)
            path = os.path.join(cacheDir, 'songs.dmp')
            f = open(path, 'wb')
            pickle.dump(songs, f, protocol=pickle.HIGHEST_PROTOCOL)
            f.close()
        except:
            xbmc.log("An error occurred saving songs")
            pass
    
# Parse URL parameters
def get_params():
    param=[]
    paramstring=sys.argv[2]
    xbmc.log(paramstring)
    if len(paramstring)>=2:
        params=sys.argv[2]
        cleanedparams=params.replace('?','')
        if (params[len(params)-1]=='/'):
            params=params[0:len(params)-2]
        pairsofparams=cleanedparams.split('&')
        param={}
        for i in range(len(pairsofparams)):
            splitparams={}
            splitparams=pairsofparams[i].split('=')
            if (len(splitparams))==2:
                param[splitparams[0]]=splitparams[1]
    return param
        
# Main
grooveshark = Groveshark();

params=get_params()
mode=None
try: mode=int(params["mode"])
except: pass
id=0
try: id=int(params["id"])
except: pass
name = None
try: name=urllib.unquote_plus(params["name"])
except: pass

# Call function for URL
if mode==None:
    grooveshark.categories()
       
elif mode==MODE_SEARCH_SONGS:
    grooveshark.searchSongs()
    
elif mode==MODE_SEARCH_ALBUMS:
    grooveshark.searchAlbums()

elif mode==MODE_SEARCH_ARTISTS:
    grooveshark.searchArtists()
    
elif mode==MODE_SEARCH_ARTISTS_ALBUMS:
    grooveshark.searchArtistsAlbums(name)
    
elif mode==MODE_SEARCH_PLAYLISTS:
    grooveshark.searchPlaylists() 

elif mode==MODE_POPULAR_SONGS:
    grooveshark.popularSongs()
    
elif mode==MODE_ARTIST_POPULAR:
    grooveshark.artistPopularSongs()

elif mode==MODE_FAVORITES:
    grooveshark.favorites()

elif mode==MODE_PLAYLISTS:
    grooveshark.playlists()
    
elif mode==MODE_SONG_PAGE:
    try: page=urllib.unquote_plus(params["page"])
    except: pass
    try: label=urllib.unquote_plus(params["label"])
    except: pass
    grooveshark.songPage(page, label, id, name)

elif mode==MODE_SONG:
    try: album=urllib.unquote_plus(params["album"])
    except: pass
    try: artist=urllib.unquote_plus(params["artist"])
    except: pass
    try: coverart=urllib.unquote_plus(params["coverart"])
    except: pass
    song = grooveshark.songItem(id, name, album, artist, coverart)
    grooveshark.playSong(song)

elif mode==MODE_ARTIST:
    grooveshark.artist(id)
    
elif mode==MODE_ALBUM:
    grooveshark.album(id)
    
elif mode==MODE_PLAYLIST:
    version = 2
    try: version=urllib.unquote_plus(params["version"])
    except: pass
    grooveshark.playlist(id, name, version)
    
elif mode==MODE_FAVORITE:
    grooveshark.favorite(id)
    
elif mode==MODE_UNFAVORITE:
    try: prevMode=int(urllib.unquote_plus(params["prevmode"]))
    except:
        prevMode = 0
    grooveshark.unfavorite(id, prevMode)

elif mode==MODE_SIMILAR_ARTISTS:
    grooveshark.similarArtists(id)

elif mode==MODE_MAKE_PLAYLIST:
    grooveshark.makePlaylist(id, name)
    
elif mode==MODE_REMOVE_PLAYLIST:
    grooveshark.removePlaylist(id, name)    

elif mode==MODE_RENAME_PLAYLIST:
    grooveshark.renamePlaylist(id, name)    

elif mode==MODE_REMOVE_PLAYLIST_SONG:
    try: playlistID=urllib.unquote_plus(params["playlistid"])
    except: pass
    grooveshark.removePlaylistSong(playlistID, name, id)

elif mode==MODE_ADD_PLAYLIST_SONG:
    grooveshark.addPlaylistSong(id)        
    
if mode < MODE_SONG:
    xbmcplugin.endOfDirectory(int(sys.argv[1]))
